@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 #
-# Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
-# Yokogawa Electric Corporation.
+# Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010
+# NTT Advanced Technology, Yokogawa Electric Corporation.
 # All rights reserved.
 # 
 # Redistribution and use of this software in source and binary
@@ -43,12 +43,18 @@
 # POSSIBILITY OF SUCH DAMAGE.
 #
 # $TAHI: koi/bin/autorun/autorun.pl,v 1.2 2006/06/26 01:12:28 akisada Exp $
+#
+# $Id: autorun.pl,v 1.4 2010/07/22 12:44:40 velo Exp $
+#
 
 use Digest::MD5;
 
 use File::Basename;
 use Config;
 use Pod::Html;
+use YAML;
+use YAML::Dumper;
+
 
 sub main();
 sub printHeader();
@@ -68,21 +74,30 @@ sub printReport();
 sub parseIndex();
 sub makeFrame();
 sub printSummary();
+sub dump_yaml();
+sub pathinit(@);
+sub searchPath($$);
 
 sub prErr($);
 
 
+# global variables
+my @status = ();
+my @ARGV_INDEXGEN = @ARGV;
+{
+	# refill ' for -title option
+	# @ARGV_INDEXGEN is used as arguments for indexgen
+	foreach my $elm (@ARGV_INDEXGEN) {
+		$elm =~ s/^-title=(.+)$/-title='$1'/g;
+	}
+}
+
 main();
 # NOTREACHED
 
 END {
 	if($mkindex) {
-		if($opt_tiny != 0) {
-			makeFrame();
-			printSummary();
-		}
-
-		printReport()
+		dump_yaml();
 	}
 
 	if($dogen){
@@ -95,7 +110,8 @@ sub main()
 {
 	my $_function_ = 'main';
 	$| = 1;
-	#pathinit();
+	pathinit();
+
 	parseArgs();
 
 	if (-e "index.html" && !$opt_f) {
@@ -139,7 +155,7 @@ sub main()
 
 	# get tool & test version
 	my $dummy;
-	($dummy, $VERTOOLS) = ('$Name: REL_1_1_1 $' =~ /\$(Name): (.*) \$/ );
+	($dummy, $VERTOOLS) = ('$Name: REL_2_2_0 $' =~ /\$(Name): (.*) \$/ );
 	unless ($VERTOOLS) { # if undef or blank (" ")
 		$VERTOOLS = 'undefined';
 	}
@@ -169,12 +185,8 @@ sub main()
 		}
 
 		if ($dotest) {
-			if ($opt_tiny != 0) {
-				makeFrame();
-				printSummary();
-			}
+			dump_yaml();
 
-			printReport();
 			print "========== TEST $current{number} ==========\n";
 			$rc = execScript( $current{script},
 					  $current{packet},
@@ -1039,6 +1051,67 @@ sub parseArgs() {
 	$Title= $opt_title ? $opt_title : "IPv6 Conformance Test";
 }
 
+my $V6EVALBINPREFIX = "/usr/local/v6eval";
+my $V6EVALROOT;
+my $V6EVALBIN;
+my $V6EVALETC;
+my $V6EVALINC;
+
+sub pathinit(@)
+{
+	my($path) = @_;
+
+	($SeqName,$SeqDir,$SeqSuffix) = fileparse($0,'.seq');
+	$V6EVALPREFIX= "/usr/local/v6eval";
+	$V6EVALROOT  = $path            || $ENV{V6EVALROOT} || $V6EVALPREFIX;
+	$V6EVALBIN   = $ENV{V6EVALBIN}  || "$V6EVALROOT/bin/";
+	$V6EVALETC   = $ENV{V6EVALETC}  || "./:$V6EVALROOT/etc/";
+	$V6EVALINC   = $ENV{V6EVALINC}  || "$SeqDir:./:$V6EVALROOT/include/";
+}
+
+sub searchPath($$)
+{
+	my($path, $filename) = @_;
+
+	my $fullname = "";
+	if ($filename =~ m!.*/.+!) {
+		$fullname = $filename;
+	}
+	my @paths = split(/:/,($path));
+	foreach (@paths) {
+		my $tmpname = $_ . "/" . $filename;
+		if( $fullname eq "") {
+			$fullname = $tmpname if(-r $tmpname);
+		}
+	}
+	return($fullname);
+}
+
+sub dump_yaml()
+{
+	my $data = {
+		'VERTESTS'	=> $VERTESTS,
+		'VERTOOLS'	=> $VERTOOLS,
+		'StartTime'	=> $StartTime,
+		'EndTime'	=> $EndTime,
+		'results'	=> \@status,
+	};
+
+	my $yaml_file = '/tmp/indexgen.yaml';
+	my $dumper = YAML::Dumper->new;
+	local(*YAML);
+	open(YAML, '>'. $yaml_file);
+	print YAML $dumper->dump($data);
+	close(YAML);
+
+	my $cmd = searchPath($V6EVALBIN, "indexgen") || "indexgen";
+	$cmd .= ' ' . join(' ', @ARGV_INDEXGEN) ;
+	my $ret = system($cmd);
+
+	unlink($yaml_file);
+
+	return($ret);
+}
 
 sub prErr($)
 {
