@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
- * Yokogawa Electric Corporation.
+ *
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010
+ * NTT Advanced Technology, Yokogawa Electric Corporation.
  * All rights reserved.
  * 
  * Redistribution and use of this software in source and binary
@@ -42,6 +43,9 @@
  * POSSIBILITY OF SUCH DAMAGE.
  *
  * $TAHI: koi/include/koi.h,v 1.125 2007/04/06 01:32:07 akisada Exp $
+ *
+ * $Id: koi.h,v 1.7 2008/12/25 11:40:02 inoue Exp $
+ *
  */
 
 #ifndef _KOI_H_
@@ -114,11 +118,11 @@ int kDataRegist(short, unsigned char, long, unsigned char *, struct timeval);
 int kDataGetDataByDataid(long, MsgData *, unsigned char *);
 int kDataGetDataBySocketid(short, MsgData *, unsigned char *);
 #if SUPPORT_TLS
-int kTLSInitialize(int, int, int, char *, char *, char *, char *, int);
-int kTLSAccept(int, void **, void **);
+int kTLSInitialize(int,int,int,char *,char *,char *,char *,int,int,int,int,char *);
+int kTLSAccept(int handle,void **tlsssl,void **tlssession);
 void *kTLSConnect(int, void *);
-int kTLSSend(void *, char *, int);
-int kTLSRecv(void *, char *, int);
+int kTLSSend(void *tlsssl,char *buff,int buffsize);
+int kTLSRecv(void *tlsssl,char *buff,int buffsize);
 int kTLSClear(int);
 bool kIsTLSInitialize(void);
 void *kGetTLSSession(void *);
@@ -149,6 +153,7 @@ extern MsgData *g_msg_start_ptr;	/* msg first data pointer */
 
 bool	kLogInit(const char *, int);
 bool	kLogWrite(int, const char *, ...);
+bool	kIsLogLevel(int);
 void	kLogFinalize(void);
 
 
@@ -320,7 +325,7 @@ int kSocketRegistDataSocketInfo(
 	struct sockaddr_storage d_addr,
 	int (*)(long, unsigned char*,	unsigned char**, short*),
 	short,
-	unsigned char);
+	unsigned char,int,void*);
 
 /* use in kData  */
 int kSocketGetSIBySocketId(short, SocketInfo *, UnixdInfo *,
@@ -336,7 +341,8 @@ int kSocketCheckNewSocket(unsigned short, struct in6_addr,
 
 int icmp_recv(int, const unsigned char*,
 	      struct sockaddr*, socklen_t,
-	      struct sockaddr*, socklen_t);
+	      struct sockaddr*, socklen_t,
+	      char *, int);
 
 
 
@@ -402,6 +408,7 @@ int kRecvTimerProc(void);
 #define CM_CM_SUB_TLS_SETUP	(0x0009)
 #define CM_CM_SUB_TLS_CLEAR	(0x000A)
 #endif	/* SUPPORT_TLS */
+#define	CM_CM_SUB_PARSER_ATTACH	(0x000B)
 
 /* command(2byte) */
 #define	CM_CM_SIG_CONNEC_REQ	\
@@ -446,6 +453,10 @@ int kRecvTimerProc(void);
 #define CM_CM_SIG_TLS_CLEAR_ACK	\
 	((CM_CM_CATEG_SIG)	| (CM_CM_SUB_TLS_CLEAR)	| (CM_CM_TYPE_ACK))
 #endif	/* SUPPORT_TLS */
+#define	CM_CM_SIG_PARSER_ATTACH_REQ	\
+	((CM_CM_CATEG_SIG)	| (CM_CM_SUB_PARSER_ATTACH)	| (CM_CM_TYPE_REQ))
+#define	CM_CM_SIG_PARSER_ATTACH_ACK	\
+	((CM_CM_CATEG_SIG)	| (CM_CM_SUB_PARSER_ATTACH)	| (CM_CM_TYPE_ACK))
 
 /* result(2byte)*/
 #define	CM_RESULT_OK			(0x0000)
