@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
- * Yokogawa Electric Corporation.
+ *
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010
+ * NTT Advanced Technology, Yokogawa Electric Corporation.
  * All rights reserved.
  * 
  * Redistribution and use of this software in source and binary
@@ -42,6 +43,9 @@
  * POSSIBILITY OF SUCH DAMAGE.
  *
  * $TAHI: koi/lib/kData/kData.c,v 1.39 2007/04/06 01:32:07 akisada Exp $
+ *
+ * $Id: kData.c,v 1.3 2008/06/03 07:39:55 akisada Exp $
+ *
  */
 
 #include <stdio.h>
@@ -790,7 +794,7 @@ kDataGetDataBySocketid(short socketid, MsgData *msginfo, unsigned char *data)
 	msginfo->msg_recvflg               = MSG_RECVFLG_READ;
 	current_data_position->msg_recvflg = MSG_RECVFLG_READ;
 
-	kLogWrite(L_CMD,
+	kLogWrite(L_INFO,
 		"%s: dataid[%ld] socketid[%d] socketmode[%d] "
 		"recvflg[%d] family[%d] datalen[%ld] data[%100.100s]",
 		__FUNCTION__, msginfo->msg_dataid,
