@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
- * Yokogawa Electric Corporation.
+ *
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010
+ * NTT Advanced Technology, Yokogawa Electric Corporation.
  * All rights reserved.
  * 
  * Redistribution and use of this software in source and binary
@@ -42,8 +43,18 @@
  * POSSIBILITY OF SUCH DAMAGE.
  *
  * $TAHI: koi/lib/kDispatcher/kDispatcher.c,v 1.59 2007/04/05 07:54:16 akisada Exp $
+ *
+ * $Id: kDispatcher.c,v 1.14 2009/06/16 08:43:19 velo Exp $
+ *
  */
 
+/*
+  08/4/18 SocketInfoDump: change display information
+  08/3/18 read_unixd_data: create
+          udpaccept,process_si_type_unixd: add setsockopt(SO_SNDBUF,SO_RCVBUF)
+          process_si_type_unixd_data: call read_unixd_data in place of read
+*/
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -108,6 +119,7 @@ static bool process_command_flush_req(int, short, char *, long);
 static bool process_command_tls_setup_req(int, short, char*, long);
 static bool process_command_tls_clear_req(int, short, char*, long);
 #endif	/* SUPPORT_TLS */
+static bool process_command_parser_attach_req(int, short, char *, long);
 
 /* debug function */
 static void SocketInfoDump(void);
@@ -136,8 +148,9 @@ static char *COMMAND_NAME[] = {
 int
 kDispatcherProc(int handle, unsigned char type)
 {
-	kLogWrite(L_INFO, "%s: start socketfd[%d] type[%d]----------------",
-		__FUNCTION__, handle, type);
+  static char *Type[]={"UNIX","UNIX-DATA","UDP-DATA","TCP-DATA","UDP","TCP"};
+
+	kLogWrite(L_DEBUG,"%s: socketfd[%d] pkt-type[%s] <=", __FUNCTION__, handle, Type[type-1]);
 
 	switch(type) {
 		case SI_TYPE_UNIX:
@@ -160,6 +173,7 @@ kDispatcherProc(int handle, unsigned char type)
 		default:
 			return(false);
 	}
+	kLogWrite(L_DEBUG,"%s: socketfd[%d] pkt-type[%s] =>", __FUNCTION__, handle, Type[type-1]);
 
 	return(true);
 }
@@ -225,6 +239,8 @@ udpaccept(int fd,
 	const int on = 1;
 	struct cmsghdr *cmsgptr = NULL;
 	unsigned char dstaddr[32];
+	int sndbuf = 0xffff;
+	int rcvbuf = 0xffff;
 
 	memset(&iov, 0, sizeof(iov));
 	memset(control, 0, sizeof(control));
@@ -256,6 +272,9 @@ udpaccept(int fd,
 
 		return(-1);
 	}
+	kLogWrite(L_SOCKET,
+		"%s: handle[%d] recvmsg() success -- %ld bytes",
+			__FUNCTION__, fd, *buflen);
 
 	for (cmsgptr = CMSG_FIRSTHDR(&msg); cmsgptr != NULL; cmsgptr = CMSG_NXTHDR(&msg, cmsgptr)) {
 	  if (cmsgptr->cmsg_level == IPPROTO_IP && cmsgptr->cmsg_type == DSTADDR_SOCKOPT) {
@@ -359,6 +378,22 @@ udpaccept(int fd,
 		return(-1);
 	}
 
+	if(setsockopt(udpfd,
+		      SOL_SOCKET,SO_SNDBUF, &sndbuf, sizeof(sndbuf))) {
+		close(udpfd);
+		kLogWrite(L_WARNING,
+		    "%s: setsockopt(SO_SNDBUF) fails", __FUNCTION__);
+		return(-1);
+	}
+
+	if(setsockopt(udpfd,
+		      SOL_SOCKET,SO_RCVBUF, &rcvbuf, sizeof(rcvbuf))) {
+		close(udpfd);
+		kLogWrite(L_WARNING,
+		    "%s: setsockopt(SO_RCVBUF) fails", __FUNCTION__);
+		return(-1);
+	}
+
 	if(bind(udpfd, (struct sockaddr *)&src, srclen) < 0) {
 		perror("bind");
 		close(udpfd);
@@ -378,7 +413,7 @@ udpaccept(int fd,
 		return(-1);
 	}
 
-	kLogWrite(L_INFO, "%s: return new socket fd[%d]", __FUNCTION__, udpfd);
+	kLogWrite(L_SOCKET, "%s: UDP accept new socket fd[%d]", __FUNCTION__, udpfd);
 
 	return(udpfd);
 }
@@ -402,6 +437,7 @@ process_si_type_listen(int handle, unsigned char type)
 	int	si_type = 0;
 	int	newid = 0;
 	unsigned char	af = 0;
+	void *tlsssl=NULL,*tlssession=NULL;
 
 	struct sockaddr_storage	src;
 	struct sockaddr_storage	dst;
@@ -525,6 +561,15 @@ process_si_type_listen(int handle, unsigned char type)
 	kdbg("/tmp/dbg_parser.txt", "parser: %p\n", parser);
 #endif	/* DBG_PARSER */
 
+#ifdef SUPPORT_TLS
+	/* TLS auth Handshake */
+	if(socketinfo.si_tls_mode){
+	  if(kTLSAccept(data_socket_handle,&tlsssl,&tlssession)){
+		kLogWrite(L_CMD, "%s: kTLSAccept TLS connection accesp error. abort.",__FUNCTION__);
+		return(RETURN_NG);
+	  }
+	}
+#endif	
 	/* regist child socoket */
 	si_type = (type == SI_TYPE_LISTEN_TCP) 
 		? SI_TYPE_DATA : SI_TYPE_UDP;
@@ -540,7 +585,7 @@ process_si_type_listen(int handle, unsigned char type)
 		dst,
 		parser,
 		socketinfo.si_socketid,
-		af);
+		af,socketinfo.si_tls_mode,tlsssl);
 
 	/* select */
 	kSocketSelectOperation(data_socket_handle, SELECT_IN);
@@ -551,7 +596,7 @@ process_si_type_listen(int handle, unsigned char type)
 		struct timeval	current_time;
 		gettimeofday(&current_time, NULL);
 		if((kDataRegist(newid, MSG_SOCKETMODE_RECV,
-				recvbufsize, recvbuf, current_time)) < 0) {
+				recvbufsize, (unsigned char *)recvbuf, current_time)) < 0) {
 			kLogWrite(L_INFO,
 				"%s: regist_data fail for handle[%d] socketid[%d]",
 				__FUNCTION__, handle, newid);
@@ -631,6 +676,7 @@ recv_ack_return(int handle, short command,
 	}
 
 	/* send back */
+	/* printf("recv_ack_return [%d]\n",write_data_size); */
 	if((write(handle, &recv_ack,
 		(sizeof(recv_ack) - (MSG_MAXDATALEN) + write_data_size))) < 0) {
 
@@ -638,7 +684,6 @@ recv_ack_return(int handle, short command,
 			"%s: write() error. errno is %d.", __FUNCTION__, errno);
 		return(RETURN_NG);
 	}
-
 	kLogWrite(L_INFO, "%s: write() success", __FUNCTION__);
 
 	return(RETURN_OK);
@@ -777,14 +822,18 @@ sockinfo_ack_return(int handle, short command, short request_num, short result,
 		 */
 		if((socketinfo->si_type == SI_TYPE_DATA) ||
 		   (socketinfo->si_type == SI_TYPE_LISTEN_TCP)) {
-			sockinfo_ack.protocol = CM_PROT_TCP;
-
+			sockinfo_ack.protocol = socketinfo->si_tls_mode ? CM_PROT_TLS : CM_PROT_TCP;
 			kLogWrite(L_DEBUG, "%s: type is TCP", __FUNCTION__);
 		} else if((socketinfo->si_type == SI_TYPE_UDP) ||
 			(socketinfo->si_type == SI_TYPE_LISTEN_UDP)) {
 			sockinfo_ack.protocol = CM_PROT_UDP;
 
 			kLogWrite(L_DEBUG, "%s: type is UDP", __FUNCTION__);
+		}
+		else if (socketinfo->si_type == SI_TYPE_RAW) {
+			sockinfo_ack.protocol = CM_PROT_ICMP;
+
+			kLogWrite(L_DEBUG, "%s: type is ICMP", __FUNCTION__);
 		} else {
 			kLogWrite(L_WARNING,
 				"%s: socoketid[%d].requested socket's type is UNIX.abort",
@@ -856,11 +905,23 @@ sockinfo_ack_return(int handle, short command, short request_num, short result,
 			       sizeof(sockinfo_ack.s_addr));
 			kLogWrite(L_DEBUG, "%s: s_port s_addr avaliable", __FUNCTION__);
 		}
+		else if (socketinfo->si_type == SI_TYPE_RAW) {
+			sockinfo_ack.d_port	= htons(datasocketinfo->di_d_port);
+			memcpy(&(sockinfo_ack.s_addr), &(datasocketinfo->di_s_addr),
+			       sizeof(sockinfo_ack.s_addr));
+			memcpy(&(sockinfo_ack.d_addr), &(datasocketinfo->di_d_addr),
+			       sizeof(sockinfo_ack.d_addr));
+			kLogWrite(L_DEBUG,
+				"%s: s_addr d_addr avaliable", __FUNCTION__);
+		}
 
 		/* interfaceid and frameid */
 		sockinfo_ack.frameid		= htons(socketinfo->si_peyload_type);
 		sockinfo_ack.interfaceid
-			= htons(atoi(&(socketinfo->si_linkname[IF_KEYSIZE])));
+			= htons(atoi((const char *)&(socketinfo->si_linkname[IF_KEYSIZE])));
+		sockinfo_ack.tlsmode		= htons(socketinfo->si_tls_mode);
+		sockinfo_ack.tlsssl		= htonl((u_long)socketinfo->si_tls_ssl_ptr);
+		sockinfo_ack.tlssession	= htonl((u_long)kGetTLSSession(socketinfo->si_tls_ssl_ptr));
 
 		kLogWrite(L_DEBUG, "%s: result OK", __FUNCTION__);
 	}
@@ -965,8 +1026,6 @@ flush_ack_return(int handle, short command, short request_num, short result)
 	return(true);
 }
 
-
-
 #ifdef SUPPORT_TLS
 /*--------------------------------------------------------------------*
  * static bool tls_setup_ack_return(int, short, short, short);        *
@@ -1045,8 +1104,6 @@ tls_clear_ack_return(int handle, short command, short request_num, short result)
 }
 #endif	/* SUPPORT_TLS */
 
-
-
 /* process_si_type_unixd */
 static int
 process_si_type_unixd(int handle)
@@ -1055,6 +1112,9 @@ process_si_type_unixd(int handle)
 	int		data_socket_handle;
 	struct sockaddr_un 	serveradd;
 
+	int sndbuf = 0xffff;
+	int rcvbuf = 0xffff;
+
 	/*
 	 * 1  accept connection
 	 * 2  into the select list
@@ -1063,27 +1123,84 @@ process_si_type_unixd(int handle)
 	 * 5  copy the data
 	 */
 
-	kLogWrite(L_INFO, "%s: start", __FUNCTION__);
-
 	/* accept */
 	socklen = sizeof(struct sockaddr_un);
 	data_socket_handle = accept(handle,
 				    (struct sockaddr*)&serveradd,
 				    &socklen);
 
+	kLogWrite(L_SOCKET," Unix domain sock accept[%d] parent[%d]",data_socket_handle,handle);
+
+	if(setsockopt(data_socket_handle,
+		      SOL_SOCKET,SO_SNDBUF, &sndbuf, sizeof(sndbuf))) {
+	  kLogWrite(L_WARNING,
+		    "%s: setsockopt(SO_SNDBUF) fails", __FUNCTION__);
+	}
+
+	if(setsockopt(data_socket_handle,
+		      SOL_SOCKET,SO_RCVBUF, &rcvbuf, sizeof(rcvbuf))) {
+  
+	  kLogWrite(L_WARNING,
+		    "%s: setsockopt(SO_RCVBUF) fails", __FUNCTION__);
+	}
+
 	/* regist child socoket */
 	kSocketRegistUnixdSocketInfo(SI_TYPE_UNIX_DATA,
 				     data_socket_handle,
-				     serveradd.sun_path);
+				     (unsigned char *)serveradd.sun_path);
 
 	/* select() */
 	kSocketSelectOperation(data_socket_handle, SELECT_IN);
 
-	kLogWrite(L_INFO, "%s: ok. done.", __FUNCTION__);
-
 	return(RETURN_OK);
 }
 
+static int
+read_unixd_data(int handle,char *buff,int bufsize)
+{
+  int datalen,pktlen,len;
+  short command;
+
+  datalen = read(handle, buff, sizeof(command));
+  if(datalen < sizeof(command)) goto ERR;
+
+  command = ntohs(*((short*)buff));
+
+  switch( command ){
+  case CM_CM_SIG_CONNEC_REQ:
+    datalen += read(handle, buff+datalen, sizeof(CONNEC_REQ)-datalen-MSG_MAXDATALEN);
+    if(datalen < sizeof(CONNEC_REQ)-MSG_MAXDATALEN) goto ERR;
+    pktlen = ntohl( ((CONNEC_REQ*)buff)->datalen );
+    while(datalen<sizeof(CONNEC_REQ)-MSG_MAXDATALEN+pktlen){
+      len = read(handle, buff+datalen, sizeof(CONNEC_REQ)-datalen);
+      /* printf("read_unixd_data [%d][%d]\n",len,datalen); */
+      if(len <= 0) goto ERR;
+      datalen += len;
+    }
+    break;
+  case CM_CM_SIG_SEND_REQ:
+    datalen += read(handle, buff+datalen, sizeof(SEND_REQ)-datalen-MSG_MAXDATALEN);
+    if(datalen < sizeof(SEND_REQ)-MSG_MAXDATALEN) goto ERR;
+    pktlen = ntohl( ((SEND_REQ*)buff)->datalen );
+    while(datalen<sizeof(SEND_REQ)-MSG_MAXDATALEN+pktlen){
+      len = read(handle, buff+datalen, sizeof(SEND_REQ)-datalen);
+      /* printf("read_unixd_data [%d][%d]\n",len,datalen); */
+      if(len <= 0) goto ERR;
+      datalen += len;
+    }
+    break;
+  default:
+    datalen += read(handle, buff+datalen, bufsize-datalen);
+    break;
+  }
+  /* printf("read_unixd_data OK len[%d]\n",datalen); */
+  return datalen;
+
+ ERR:
+  /* printf("read_unixd_data too short len[%d]\n",datalen); */
+  return datalen;
+}
+
 static int
 process_si_type_unixd_data(int handle)
 {
@@ -1094,9 +1211,12 @@ process_si_type_unixd_data(int handle)
 	short		command;
 	short		request_num;
 	char		*current_position;
+	static   char   *Service[]={"ConnectSend","Send","Recv","Close","StartRecv","ConnectInfo","DataInfo","Clear",
+				    "TLS-Setup","TLS-Clear","----"};
 
 	/* read() */
-	datalen = read(handle, databuf, CM_MAX_CM_LENGTH);
+	/* datalen = read(handle, databuf, CM_MAX_CM_LENGTH); */
+	datalen = read_unixd_data(handle, databuf, CM_MAX_CM_LENGTH);
 	if(datalen < 0) {
 		kLogWrite(L_WARNING,
 			"%s: read fail for handle[%d]. errno is [%d]",
@@ -1116,7 +1236,7 @@ process_si_type_unixd_data(int handle)
 		}
 
 		kSocketClose(socketinfo.si_socketid);
-		kLogWrite(L_INFO,
+		kLogWrite(L_SOCKET,
 			"%s: read_size0 for socketid[%d] handle[%d], so closed",
 			__FUNCTION__, socketinfo.si_socketid, handle);
 
@@ -1139,13 +1259,13 @@ process_si_type_unixd_data(int handle)
 	command			= ntohs(*((short*)current_position));
 	request_num		= ntohs(*((short*)current_position+1));
 
-	kLogWrite(L_DEBUG, "%s: command is [%x]", __FUNCTION__, command);
+	kLogWrite(L_CMD," service command <=[%s] leng(%ld)", Service[(command & 0xf)-1],datalen);
 
 	switch (command) {
 	case CM_CM_SIG_CONNEC_REQ:
 		if(process_command_connect_req(handle, request_num,
 						databuf, datalen) < 0) {
-			return(RETURN_NG);
+		        goto ERR;
 		}
 
 		break;
@@ -1153,7 +1273,7 @@ process_si_type_unixd_data(int handle)
 	case CM_CM_SIG_SEND_REQ:
 		if(process_command_send_req(handle, request_num,
 					     databuf, datalen) < 0) {
-			return(RETURN_NG);
+		        goto ERR;
 		}
 
 		break;
@@ -1161,7 +1281,7 @@ process_si_type_unixd_data(int handle)
 	case CM_CM_SIG_RECV_REQ:
 		if(process_command_recv_req(handle, request_num,
 					     databuf, datalen) < 0) {
-			return(RETURN_NG);
+		        goto ERR;
 		}
 
 		break;
@@ -1169,7 +1289,7 @@ process_si_type_unixd_data(int handle)
 	case CM_CM_SIG_CLOSE_REQ:
 		if(process_command_close_req(handle, request_num,
 					      databuf, datalen) < 0) {
-			return(RETURN_NG);
+		        goto ERR;
 		}
 
 		break;
@@ -1177,7 +1297,7 @@ process_si_type_unixd_data(int handle)
 	case CM_CM_SIG_WAIT_REQ:
 		if(process_command_wait_req(handle, request_num,
 					     databuf, datalen) < 0) {
-			return(RETURN_NG);
+		        goto ERR;
 		}
 
 		break;
@@ -1185,7 +1305,7 @@ process_si_type_unixd_data(int handle)
 	case CM_CM_SIG_SOCKINFO_REQ:
 		if(process_command_sockinfo_req(handle, request_num,
 						 databuf, datalen) < 0) {
-			return(RETURN_NG);
+		        goto ERR;
 		}
 
 		break;
@@ -1193,7 +1313,7 @@ process_si_type_unixd_data(int handle)
 	case CM_CM_SIG_DATAINFO_REQ:
 		if(process_command_datainfo_req(handle, request_num,
 						 databuf, datalen) < 0) {
-			return(RETURN_NG);
+		        goto ERR;
 		}
 
 		break;
@@ -1201,7 +1321,7 @@ process_si_type_unixd_data(int handle)
 	case CM_CM_SIG_FLUSH_REQ:
 		if(process_command_flush_req(handle, request_num,
 					      databuf, datalen)) {
-			return(RETURN_NG);
+		        goto ERR;
 		}
 
 		break;
@@ -1210,8 +1330,7 @@ process_si_type_unixd_data(int handle)
 	case CM_CM_SIG_TLS_SETUP_REQ:
 		if(process_command_tls_setup_req(handle,
 			request_num, databuf, datalen)) {
-
-			return(RETURN_NG);
+		        goto ERR;
 		}
 
 		break;
@@ -1219,19 +1338,30 @@ process_si_type_unixd_data(int handle)
 	case CM_CM_SIG_TLS_CLEAR_REQ:
 		if(process_command_tls_clear_req(handle,
 			request_num, databuf, datalen)) {
-
-			return(RETURN_NG);
+		        goto ERR;
 		}
 
 		break;
 #endif	/* SUPPORT_TLS */
 
+	case CM_CM_SIG_PARSER_ATTACH_REQ:
+		if(process_command_parser_attach_req(handle, request_num,
+					      databuf, datalen)) {
+		        goto ERR;
+		}
+
+		break;
+
 	default:
-		kLogWrite(L_ERROR, "%s: invalid command[%x]", __FUNCTION__, command);
+		kLogWrite(L_ERROR, "%s: invalid service command <=[%x]", __FUNCTION__, command);
 		return(RETURN_NG);
 	}
-
+	kLogWrite(L_CMD," service command =>[%s] OK", Service[(command & 0xf)-1]);
 	return(RETURN_OK);
+
+	ERR:
+	kLogWrite(L_CMD," service command =>[%s] NG", Service[(command & 0xf)-1]);
+	return(RETURN_NG);
 }
 
 
@@ -1299,7 +1429,6 @@ process_command_connect_req(
 	} else if(connec_req.protocol == CM_PROT_TLS) {
 		if(!kIsTLSInitialize()) {
 			kLogWrite(L_WARNING, "%s: TLS not Initialize yet", __FUNCTION__);
-
 			connec_send_ack_return(handle, CM_CM_SIG_CONNEC_ACK, request_num,
 				CM_RESULT_SIG_TLS_NOINIT, 0, 0, timestamp);
 			return(RETURN_NG);
@@ -1311,10 +1440,10 @@ process_command_connect_req(
 #endif	/* SUPPORT_TLS */
 	}
 
-	snprintf(linkname_temp_char, sizeof(linkname_temp_char),
+	snprintf((char *)linkname_temp_char, sizeof(linkname_temp_char),
 		"%d", connec_req.interfaceid);
-	strcat(linkname, IF_KEYWORD);
-	strcat(linkname, linkname_temp_char);
+	strcat((char *)linkname, IF_KEYWORD);
+	strcat((char *)linkname, (const char *)linkname_temp_char);
 
 	/* check whether create new socket or re-use socket */
 	if((kSocketCheckNewSocket(connec_req.s_port, connec_req.s_addr,
@@ -1617,10 +1746,10 @@ process_command_wait_req(int handle,
 	wait_req.s_port			= ntohs(wait_req.s_port);
 	wait_req.flags			= ntohs(wait_req.flags);
 
-	snprintf(linkname_temp_char,
+	snprintf((char *)linkname_temp_char,
 		sizeof(linkname_temp_char), "%d", wait_req.interfaceid);
-	strcat(linkname, IF_KEYWORD);
-	strcat(linkname, linkname_temp_char);
+	strcat((char *)linkname, IF_KEYWORD);
+	strcat((char *)linkname, (const char *)linkname_temp_char);
 
 	/* data preparation */
 	if(wait_req.protocol == CM_PROT_TCP) {
@@ -1676,8 +1805,6 @@ process_command_wait_req(int handle,
 
 	}
 
-	kLogWrite(L_CMD, "%s: open_wait_socket success", __FUNCTION__);
-
 	close_wait_ack_return(handle,
 		CM_CM_SIG_WAIT_ACK, request_num, CM_RESULT_OK, socketid);
 
@@ -1731,8 +1858,6 @@ process_command_sockinfo_req(int handle,
 		return(RETURN_NG);
 	}
 
-	kLogWrite(L_CMD, "%s: get_si_by_socketid success", __FUNCTION__);
-
 	sockinfo_ack_return(handle, CM_CM_SIG_SOCKINFO_ACK, request_num,
 		CM_RESULT_OK, (SocketInfo *)&socketinfo,
 		(DatasocketInfo *)&datasocketinfo, (ListenportInfo *)&listenportinfo);
@@ -1782,8 +1907,6 @@ process_command_datainfo_req(int handle,
 		return(RETURN_NG);
 	}
 
-	kLogWrite(L_CMD, "%s: kDataGetDataByDataid() success", __FUNCTION__);
-
 	datainfo_ack_return(handle, CM_CM_SIG_DATAINFO_ACK, request_num,
 		CM_RESULT_OK, &recvmsg);
 
@@ -1819,12 +1942,18 @@ process_command_tls_setup_req(int handle,
 	tls_setup_req.initialmode	= ntohs(tls_setup_req.initialmode);
 	tls_setup_req.ssltimeout	= ntohs(tls_setup_req.ssltimeout);
 	tls_setup_req.sslversion	= ntohs(tls_setup_req.sslversion);
+	tls_setup_req.nagle		= ntohs(tls_setup_req.nagle);
+	tls_setup_req.clientveri	= ntohs(tls_setup_req.clientveri);
+	tls_setup_req.tmprsa		= ntohs(tls_setup_req.tmprsa);
 
 	/* TLS initialize */
-	if(!(ret = kTLSInitialize(tls_setup_req.sessionmode,
-		tls_setup_req.initialmode, tls_setup_req.ssltimeout,
-		(char *)tls_setup_req.passwd, (char *)tls_setup_req.rootpem,
-		(char *)tls_setup_req.mypem, NULL, tls_setup_req.sslversion))) {
+	ret = kTLSInitialize(tls_setup_req.sessionmode,tls_setup_req.initialmode,tls_setup_req.ssltimeout,
+			     (char*)tls_setup_req.passwd,(char*)tls_setup_req.rootpem,
+			     (char*)tls_setup_req.mypem,(char*)tls_setup_req.dhpem,tls_setup_req.sslversion,
+			     tls_setup_req.nagle,tls_setup_req.clientveri,tls_setup_req.tmprsa,
+			     tls_setup_req.encsuit?(char*)tls_setup_req.encsuit:"");
+
+	if(ret) {
 
 		kLogWrite(L_CMD, "%s: TLS initilize fail. abort", __FUNCTION__);
 
@@ -1933,6 +2062,45 @@ process_command_flush_req(int handle,
 	return(true);
 }
 
+/*--------------------------------------------------------------------*
+ * static bool process_command_parser_attach_req(int, short, char *, long);   *
+ *--------------------------------------------------------------------*/
+/* process_command_parser_attach_req */
+static bool
+process_command_parser_attach_req(int handle,
+	short request_num, char *databuf, long datalen)
+{
+	bool		ret = 0;
+	PARSER_ATTACH_REQ	attach_req;
+	extern bool kParserSet2(int,char*,char*);
+	extern bool kParserDelete(int);
+
+	memset(&attach_req, 0, sizeof(attach_req));
+
+	kLogWrite(L_DEBUG, "%s: command is PARSER ARRATCH REQ", __FUNCTION__);
+
+ 	/*
+	 * data copy and valance from network byte order
+	 * to hostbyteorder
+	 */
+	memcpy(&attach_req, databuf, datalen);
+
+	attach_req.parserid	= ntohs(attach_req.parserid);
+	attach_req.mode	= ntohs(attach_req.mode);
+
+	if(attach_req.mode == 0)
+	  ret = !kParserSet2(attach_req.parserid,attach_req.parsernname,attach_req.modulepath);
+	else
+	  ret = !kParserDelete(attach_req.parserid);
+
+	kLogWrite(L_CMD, "%s: parser %s %s", __FUNCTION__,
+		  attach_req.mode?"detach":"attach",
+		  ret?"error":"success");
+
+	flush_ack_return(handle, CM_CM_SIG_PARSER_ATTACH_ACK, request_num, ret?CM_RESULT_SIG_NG_OTHER:CM_RESULT_OK);
+	return(true);
+}
+
 
 /* process_si_type_udp */
 static int
@@ -1991,7 +2159,7 @@ process_si_type_udp(int handle)
 		return(RETURN_NG);
 	}
 
-	kLogWrite(L_DEBUG,
+	kLogWrite(L_SOCKET,
 		"%s: handle[%d] recvfrom() success -- %ld bytes",
 			__FUNCTION__, handle, recvbufsize);
 
@@ -2090,7 +2258,7 @@ process_si_type_udp(int handle)
 
 	/* registdata */
 	if((kDataRegist(si->si_socketid, MSG_SOCKETMODE_RECV,
-			recvbufsize, recvbuf, current_time)) < 0) {
+			recvbufsize, (unsigned char *)recvbuf, current_time)) < 0) {
 		kLogWrite(L_INFO,
 			"%s: regist_data fail for handle[%d] socketid[%d]",
 			__FUNCTION__, handle, si->si_socketid);
@@ -2108,13 +2276,9 @@ process_si_type_raw(int handle)
 	SocketInfo socketinfo;
 	DatasocketInfo datasocketinfo;
 
-	/* registdata */
-	/* XXX */
-	/* use dummy data */
-	char *recvbuf = "";
-	int recvbufsize = 0;
-	/*char recvbuf[MSG_MAXDATALEN];*/
-	/*int recvbufsize = sizeof(recvbuf);*/
+	char recvbuf[MSG_MAXDATALEN];
+	int recvbufsize = sizeof(recvbuf);
+	int len = 0;
 
 	kLogWrite(L_DEBUG,
 		"%s: UDP connec sockethandle[%d] coming...",
@@ -2129,18 +2293,20 @@ process_si_type_raw(int handle)
 		return(RETURN_NG);
 	}
 
-	icmp_recv(socketinfo.si_sockethandle,
-		  socketinfo.si_linkname,
-		  (struct sockaddr *)&datasocketinfo.di_sss_addr,
-		  datasocketinfo.di_sss_addr.ss_len,
-		  (struct sockaddr *)&datasocketinfo.di_ssd_addr,
-		  datasocketinfo.di_ssd_addr.ss_len);
+	len = icmp_recv(socketinfo.si_sockethandle,
+			socketinfo.si_linkname,
+			(struct sockaddr *)&datasocketinfo.di_sss_addr,
+			datasocketinfo.di_sss_addr.ss_len,
+			(struct sockaddr *)&datasocketinfo.di_ssd_addr,
+			datasocketinfo.di_ssd_addr.ss_len,
+			recvbuf,
+			recvbufsize);
 
 	/* gettime */
 	gettimeofday(&current_time, NULL);
 
 	if((kDataRegist(socketinfo.si_socketid, MSG_SOCKETMODE_RECV,
-			recvbufsize, recvbuf, current_time)) < 0) {
+			len, (unsigned char *)recvbuf, current_time)) < 0) {
 		kLogWrite(L_INFO,
 			"%s: regist_data fail for handle[%d] socketid[%d]",
 			__FUNCTION__, handle, socketinfo.si_socketid);
@@ -2185,12 +2351,25 @@ process_si_type_tcp(int handle)
 	}
 
 	/* recv */
-	if((recvbufsize = recv(handle, recvbuf, sizeof(recvbuf), 0)) < 0) {
+#ifdef SUPPORT_TLS
+	if(vsocketinfo.si_tls_mode){
+	  /* TLS data receive */
+	  recvbufsize=kTLSRecv(vsocketinfo.si_tls_ssl_ptr,recvbuf,sizeof(recvbuf));
+	}
+	else{
+	  /* TCP data receive */
+	  recvbufsize = recv(handle, recvbuf, sizeof(recvbuf), 0);
+	}
+#else
+	/* TCP data receive */
+	recvbufsize = recv(handle, recvbuf, sizeof(recvbuf), 0);
+#endif
+
+	if( recvbufsize < 0) {
 		kLogWrite(L_WARNING, "%s: handle[%d] recv() fail. errno is [%d]",
 			__FUNCTION__, handle, errno);
 		return(RETURN_NG);
 	}
-
 	if(!recvbufsize) {
 		kCloseTimerSet(vsocketinfo.si_socketid, 0);
 
@@ -2204,14 +2383,10 @@ process_si_type_tcp(int handle)
 	kdmp("/tmp/dbg_parser.txt", "recvbuf", recvbuf, recvbufsize);
 #endif	/* DBG_PARSER */
 #endif	/* 0 */
-	kLogWrite(L_DEBUG,
-		"%s: recv() success for handle[%d]", __FUNCTION__, handle);
+	kLogWrite(L_IO, "%s: %s size[%d] socketID[%d] recv OK",__FUNCTION__,
+		  vsocketinfo.si_tls_mode?"TLS":"TCP" ,recvbufsize,vsocketinfo.si_socketid);
 
-	kLogWrite(L_DEBUG,
-		"%s: recv() size is [%ld]\n%100.100s\n",
-		__FUNCTION__, recvbufsize, recvbuf);
-
-	/* gettime */
+	/* timestamp */
 	gettimeofday(&current_time, NULL);
 
 	/*********** parsing data ****************/
@@ -2220,7 +2395,7 @@ process_si_type_tcp(int handle)
 	 */
 	memset(packet_gen_space, 0, sizeof(packet_gen_space));
 	kBufferGet(vsocketinfo.si_socketid,
-		&packet_gen_space_len, packet_gen_space);
+		&packet_gen_space_len, (unsigned char *)packet_gen_space);
 	kLogWrite(L_DEBUG,
 		"%s: left data for socketid[%d]'s size is %ld packetspace[%100.100s]",
 		__FUNCTION__, vsocketinfo.si_socketid,
@@ -2242,7 +2417,7 @@ process_si_type_tcp(int handle)
 	 *  loop and parse
 	 */
 	datasocketinfo = (DatasocketInfo*)vsocketinfo.si_uniq_info_ptr;
-	parse_current_position = packet_gen_space;
+	parse_current_position = (unsigned char  *)packet_gen_space;
 
 	/* get parser */
 	if(!kParserGet(vsocketinfo.si_peyload_type,
@@ -2283,7 +2458,7 @@ process_si_type_tcp(int handle)
 			return(RETURN_NG);
 		} else if(ret == PA_JUSTONE) {
 			/* regist data */
-			kLogWrite(L_DEBUG,
+			kLogWrite(L_PARSE,
 				"%s: just one packet there. finish parsing", __FUNCTION__);
 
 			kDataRegist(vsocketinfo.si_socketid,
@@ -2294,7 +2469,7 @@ process_si_type_tcp(int handle)
 			break;
 		} else if(ret == PA_LEFTDATA) {
 			/* regist data */
-			kLogWrite(L_DEBUG,
+			kLogWrite(L_PARSE,
 				"%s: you got one packet, repeat parsing....", __FUNCTION__);
 
 			kDataRegist(vsocketinfo.si_socketid,
@@ -2317,7 +2492,7 @@ process_si_type_tcp(int handle)
 				break;
 			}
 
-			kLogWrite(L_DEBUG,
+			kLogWrite(L_PARSE,
 				"%s: there is imcomplete data. odd data size is [%ld]. set "
 				"left_data.finish parings", __FUNCTION__, packet_gen_space_len);
 			kBufferSet(vsocketinfo.si_socketid,
@@ -2339,22 +2514,56 @@ process_si_type_tcp(int handle)
 static void
 SocketInfoDump(void)
 {
-	MsgData 	*msg;
+  const char *in6addr2string(struct in6_addr *addr, char *buf, int *buflen);
+  const char *inaddr2string(struct in_addr *addr, char *buf, int *buflen);
+
 	SocketInfo	*sock;
+	int no1=0;
+	/*
+	MsgData 	*msg;
 	int no1=0, no2;
 	char		s_addr[INET6_ADDRSTRLEN];
 	char		d_addr[INET6_ADDRSTRLEN];
+	*/
+	DatasocketInfo *info;
+	char buf[BUFSIZE];
+	socklen_t buflen = sizeof(buf);
+
+	if(!kIsLogLevel(L_CMD)) return;
 
 	sock = g_socket_start_ptr;
 	while(sock) {
-		printf("== SocketInfo [%d] ==\n", no1 ++);
+		printf("\n== SocketInfo [%d] ==\n", no1 ++);
 		printf("  Type:[%d]\n",        sock->si_type);
 		printf("  Socketid:[%d]\n",    sock->si_socketid);
 		printf("  Sockethandle:[%d]\n",sock->si_sockethandle);
 		printf("  Linkname:[%s]\n",    sock->si_linkname);
 		printf("  Status:[%d]\n",      sock->si_socket_status);
 		printf("  PayloadType:[%d]\n", sock->si_peyload_type);
+#ifdef SUPPORT_TLS
+		printf("  TLS flag:[%d]\n",    sock->si_tls_mode);
+		printf("  TLS SSL:[%x]\n",     (u_int)sock->si_tls_ssl_ptr);
+#endif
+		if(sock->si_type == SI_TYPE_UDP &&
+		   sock->si_socket_status == SI_STATUS_ALIVE &&
+		   sock->si_uniq_info_ptr
+		   ) {
+		  info=(DatasocketInfo*)sock->si_uniq_info_ptr;
+		  if( info->di_family == PF_INET6 )
+		    in6addr2string(&info->di_s_addr,buf,(int *)&buflen);
+		  else
+		    inaddr2string((struct in_addr *)&info->di_s_addr,buf,(int *)&buflen);
+		  printf("  Src addr:[%s]\n",buf);
+		  if( info->di_family == PF_INET6 )
+		    in6addr2string(&info->di_d_addr,buf,(int *)&buflen);
+		  else
+		    inaddr2string((struct in_addr *)&info->di_d_addr,buf,(int *)&buflen);
+		  printf("  Dst addr:[%s]\n",buf);
+		  printf("  Src port:[%d]\n",info->di_s_port);
+		  printf("  Dst port:[%d]\n",info->di_d_port);
+		}
 
+#if 0
 		msg=g_msg_start_ptr; no2=0;
 		while(msg) {
 			if(msg->msg_socketid == sock->si_socketid) {
@@ -2373,14 +2582,19 @@ SocketInfoDump(void)
 				printf("    S_addr:[%p]\n", s_addr);
 				printf("    D_addr:[%p]\n", d_addr);
 				printf("    Datalen:[%ld]\n", msg->msg_datalen);
-				printf("    Data:[%30.30s]\n",
-					msg->msg_data_ptr);
+				/* XXX: */
+				/* if socket is RAW socket, this value points to NULL */
+
+				  printf("    Data:[%30.30s]\n",
+				  msg->msg_data_ptr);
+
+
 				break;
 			}
 
 			msg=msg->msg_next_ptr;
 		}
-
+#endif	/* 0 */
 		sock=sock->si_next_socket_info_ptr;
 	}
 }
