@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
- * Yokogawa Electric Corporation.
+ *
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010
+ * NTT Advanced Technology, Yokogawa Electric Corporation.
  * All rights reserved.
  * 
  * Redistribution and use of this software in source and binary
@@ -42,8 +43,17 @@
  * POSSIBILITY OF SUCH DAMAGE.
  *
  * $TAHI: koi/lib/kSocket/kSocket.c,v 1.79 2007/04/05 07:54:17 akisada Exp $
+ *
+ * $Id: kSocket.c,v 1.8 2008/12/25 11:50:57 inoue Exp $
+ *
  */
 
+/*
+  08/4/18 kSocketCheckNewSocket: add dst-port for search parameters
+  08/3/18 kSocketOpen: add setsockopt(SO_RCVBUF,SO_SNDBUF)
+          kSocketOpenWaitSocket: add setsockopt(SO_RCVBUF,SO_SNDBUF)
+*/
+
 #include <errno.h>
 #include <netdb.h>
 #include <stdio.h>
@@ -98,7 +108,8 @@ char *kAddr2Str(struct sockaddr *);
 static unsigned int linkname2ifindex(const unsigned char*);
 static int icmp_send(int, const unsigned char*,
 		     struct sockaddr*, socklen_t,
-		     struct sockaddr*, socklen_t);
+		     struct sockaddr*, socklen_t,
+		     long, unsigned char*);
 
 const char *inaddr2string(struct in_addr*, char*, int*);
 const char *in6addr2string(struct in6_addr*, char*, int*);
@@ -282,6 +293,9 @@ kSocketOpen(
 	const int on = 1;
 	const short BROADCAST_MASK = 0x0001;
 
+	int sndbuf = 0xffff;
+	int rcvbuf = 0xffff;
+
 	memset(&ss,         0, sizeof(ss));
 	memset(&local_si6,  0, sizeof(local_si6));
 	memset(&local_si,   0, sizeof(local_si));
@@ -472,6 +486,24 @@ kSocketOpen(
 			continue;
 		}
 
+		if((error = setsockopt(sockfd,
+			SOL_SOCKET,SO_SNDBUF, &sndbuf, sizeof(sndbuf)))) {
+
+			kLogWrite(L_WARNING,
+				"%s: setsockopt(SO_SNDBUF) fails", __FUNCTION__);
+			close (sockfd);
+			continue;
+		}
+
+		if((error = setsockopt(sockfd,
+			SOL_SOCKET,SO_RCVBUF, &rcvbuf, sizeof(rcvbuf)))) {
+
+			kLogWrite(L_WARNING,
+				"%s: setsockopt(SO_RCVBUF) fails", __FUNCTION__);
+			close (sockfd);
+			continue;
+		}
+
 		if(multicast && (aip->ai_family == AF_INET6)) {
 			unsigned int ifindex = 0;
 
@@ -543,10 +575,9 @@ kSocketOpen(
 			"%s: connect OK. socket handle[%d]", __FUNCTION__, sockfd);
 #ifdef SUPPORT_TLS
 		if(tlsmode){
-			/* ѲǽTLSͥμ */
+			/* get re-use TLS connection */
 			kSocketTLSSession(srcRes->ai_addr, aip->ai_addr, d_port, &tlsssl);
 
-			/* TLS ͥγΩ */
 			if(!(tlsssl=kTLSConnect(sockfd,tlsssl))) {
 				perror("connect");
 				(void)close(sockfd);
@@ -613,7 +644,7 @@ kSocketOpen(
 	socketInfo->si_socketid	= g_socket_socketid;
 
 	socketInfo->si_sockethandle = sockfd;
-	strcpy(socketInfo->si_linkname, linkname);
+	strcpy((char *)socketInfo->si_linkname, (const char *)linkname);
 	socketInfo->si_socket_status = SI_STATUS_ALIVE;
 	socketInfo->si_peyload_type = peyload_type;
 #ifdef SUPPORT_TLS
@@ -772,6 +803,8 @@ kSocketOpenWaitSocket(
 	bool multicast = false;
 	bool broadcast = false;
 	const short BROADCAST_MASK = 0x0001;
+	int sndbuf = 0xffff;
+	int rcvbuf = 0xffff;
 
 #ifdef DBG_SOCKET
 	kdbg("/tmp/dbg_socket.txt", "%s\n", "initializing.");
@@ -1026,6 +1059,26 @@ kSocketOpenWaitSocket(
 			}
 		}
 
+		if(type==SI_TYPE_LISTEN_UDP){
+			if((error = setsockopt(sockfd,
+				SOL_SOCKET,SO_SNDBUF, &sndbuf, sizeof(sndbuf)))) {
+
+				kLogWrite(L_WARNING,
+					"%s: setsockopt(SO_SNDBUF) fails", __FUNCTION__);
+				close (sockfd);
+				continue;
+			}
+
+			if((error = setsockopt(sockfd,
+				SOL_SOCKET,SO_RCVBUF, &rcvbuf, sizeof(rcvbuf)))) {
+
+				kLogWrite(L_WARNING,
+					"%s: setsockopt(SO_RCVBUF) fails", __FUNCTION__);
+				close (sockfd);
+				continue;
+			}
+		}
+		
 		if(bind(sockfd, aip->ai_addr, aip->ai_addrlen) < 0) {
 			perror("bind");
 			kLogWrite(L_WARNING, "%s: bind() fail. sockfd(%d) error(%d)",
@@ -1156,9 +1209,10 @@ kSocketOpenWaitSocket(
 	socketInfo->si_socketid	= g_socket_socketid;
 
 	socketInfo->si_sockethandle	= sockfd;
-	strcpy(socketInfo->si_linkname, linkname);
+	strcpy((char *)socketInfo->si_linkname, (const char *)linkname);
 	socketInfo->si_socket_status	= SI_STATUS_ALIVE;
 	socketInfo->si_peyload_type	= peyload_type;
+	socketInfo->si_tls_mode 	= tlsmode;
 
 	if(ss.ss_family == PF_INET6) {
 		/* src */
@@ -1312,7 +1366,7 @@ kSocketOpenUnixdSocket(
 	 */
 	strcpy(path, socketpath? socketpath: _PATH_SOCKET);
 
-	kLogWrite(L_DEBUG,
+	kLogWrite(L_INIT,
 		"%s: set [%s] as unix domain path name", __FUNCTION__, path);
 
 	/*
@@ -1407,7 +1461,7 @@ kSocketOpenUnixdSocket(
 
 	/* pointer change for cast and complete uniq_info */
 	temp_unixd = (UnixdInfo*)new->si_uniq_info_ptr;
-	strcpy(temp_unixd->ui_sockpath, path);
+	strcpy((char *)temp_unixd->ui_sockpath, path);
 
 	/****************************************************
 	 * warning    UNIX domein socket "don't need" parser*
@@ -1832,13 +1886,38 @@ kSocketSend(
 		 */
 		kLogWrite(L_DEBUG, "%s: socket is tcp/data socket", __FUNCTION__);
 
-		if((send(socketinfo.si_sockethandle, data, datalen, 0)) < 0) {
+#ifdef SUPPORT_TLS
+		if(socketinfo.si_tls_mode){
+		  /* send TLS */
+		  if(socketinfo.si_tls_ssl_ptr){
+		    if( kTLSSend(socketinfo.si_tls_ssl_ptr, (char*)data, datalen) < 0 ){
+		      kLogWrite(L_WARNING,"%s: TLS send() fail. errno is [%d]",  __FUNCTION__, errno);
+		      return(RETURN_NG);
+		    }
+		    kLogWrite(L_IO, "%s: TLS size[%d] socketID[%d] send OK",__FUNCTION__,datalen,socketinfo.si_socketid);
+		  }
+		  else{
+		    kLogWrite(L_WARNING, "%s: TLS ssl is empty,so can't send socketID[%d]",__FUNCTION__,socketinfo.si_socketid);
+		  }
+		}
+		else{
+		/* send TCP */
+		  if((send(socketinfo.si_sockethandle, data, datalen, 0)) < 0) {
 			kLogWrite(L_WARNING,
 				"%s: send() fail. errno is [%d]", __FUNCTION__, errno);
 			return(RETURN_NG);
+		  }
+		  kLogWrite(L_IO, "%s: TCP send() OK", __FUNCTION__);
 		}
-
-		kLogWrite(L_DEBUG, "%s: send() OK", __FUNCTION__);
+#else
+		/* send TCP */
+		if((send(socketinfo.si_sockethandle, data, datalen, 0)) < 0) {
+		  kLogWrite(L_WARNING,
+			    "%s: send() fail. errno is [%d]", __FUNCTION__, errno);
+		  return(RETURN_NG);
+		}
+		kLogWrite(L_IO, "%s: send() OK", __FUNCTION__);
+#endif
 	}
 	else if(socketinfo.si_type == SI_TYPE_UDP) {
 		/*
@@ -1857,7 +1936,9 @@ kSocketSend(
 			  (struct sockaddr *)&datasocketinfo.di_sss_addr,
 			  datasocketinfo.di_sss_addr.ss_len,
 			  (struct sockaddr *)&datasocketinfo.di_ssd_addr,
-			  datasocketinfo.di_ssd_addr.ss_len);
+			  datasocketinfo.di_ssd_addr.ss_len,
+			  datalen,
+			  data);
 	}
 	
 
@@ -2025,10 +2106,11 @@ kSocketGetSIBySocketId(
 		}
 
 		if(!(si->si_next_socket_info_ptr)) {
+		  /*
 			kLogWrite(L_WARNING,
 				"%s: search end. not found for socketid[%d]",
 				__FUNCTION__, socketid);
-
+		  */
 			return(RETURN_NG);
 		}
 
@@ -2075,6 +2157,7 @@ kSocketGetSIBySocketId(
 		}
 		*datasocketinfo =
 			*((DatasocketInfo*)(si->si_uniq_info_ptr));
+
 		socketinfo->si_uniq_info_ptr = datasocketinfo;
 		kLogWrite(L_DEBUG, "%s: datasocketinfo data copy done", __FUNCTION__);
 
@@ -2316,7 +2399,6 @@ kSocketGetSelectedHandle(int *handle, unsigned char *type)
 
 
 #ifdef SUPPORT_TLS
-/* SSL֥ȤȰפTLSͥ */
 void 
 kSocketTLSClose(void *tlsssl)
 {
@@ -2330,15 +2412,8 @@ kSocketTLSClose(void *tlsssl)
 	}
 }
 
-
-
-/*
- * 襢ɥ쥹ݡȡɥ쥹
- * פTLSͥ򸡺
- */
 int 
-kSocketTLSSession(struct sockaddr *src,
-	struct sockaddr *dst, int port, void **tlsssl)
+kSocketTLSSession(struct sockaddr *src,	struct sockaddr *dst, int port, void **tlsssl)
 {
 	struct sockaddr_in  *src4,*dst4;
 	struct sockaddr_in6 *src6,*dst6;
@@ -2347,42 +2422,35 @@ kSocketTLSSession(struct sockaddr *src,
 
 	if(src->sa_family == AF_INET){
 		src4=(struct sockaddr_in*)src;dst4=(struct sockaddr_in*)dst;
-
 		while(current_position){
-			if(current_position->si_tls_mode &&
-				current_position->si_tls_ssl_ptr &&
-				current_position->si_type==SI_TYPE_DATA &&
-				(info=(DatasocketInfo*)current_position->si_uniq_info_ptr)) {
-
-				if((info->di_d_port == port || info->di_s_port == port) && 
-					!memcmp(&info->di_s_addr, &src4->sin_addr, 4) &&
-					!memcmp(&info->di_d_addr, &dst4->sin_addr, 4)) {
-
-					*tlsssl=current_position->si_tls_ssl_ptr;
-					return(RETURN_OK);
-				}
-			}
-
-			current_position = current_position->si_next_socket_info_ptr;
+		  if(current_position->si_tls_mode &&
+		     current_position->si_tls_ssl_ptr &&
+		     current_position->si_type==SI_TYPE_DATA &&
+		     (info=(DatasocketInfo*)current_position->si_uniq_info_ptr)) {
+		    if((info->di_d_port == port || info->di_s_port == port) && 
+		       !memcmp(&info->di_s_addr, &src4->sin_addr, 4) &&
+		       !memcmp(&info->di_d_addr, &dst4->sin_addr, 4)) {
+		      *tlsssl=current_position->si_tls_ssl_ptr;
+		      return(RETURN_OK);
+		    }
+		  }
+		  current_position = current_position->si_next_socket_info_ptr;
 		}
 	} else {
 		src6 = (struct sockaddr_in6*)src;dst6=(struct sockaddr_in6*)dst;
-
 		while(current_position){
-			if(current_position->si_tls_mode &&
-				current_position->si_tls_ssl_ptr &&
-				current_position->si_type == SI_TYPE_DATA &&
-				(info=(DatasocketInfo*)current_position->si_uniq_info_ptr)) {
-
-				if((info->di_d_port == port || info->di_s_port == port) && 
-					!memcmp(&info->di_s_addr, &src6->sin6_addr, 16) &&
-					!memcmp(&info->di_d_addr, &dst6->sin6_addr, 16)) {
-						*tlsssl=current_position->si_tls_ssl_ptr;
-						return(RETURN_OK);
-				}
-   			}
-
-			current_position = current_position->si_next_socket_info_ptr;
+		  if(current_position->si_tls_mode &&
+		     current_position->si_tls_ssl_ptr &&
+		     current_position->si_type == SI_TYPE_DATA &&
+		     (info=(DatasocketInfo*)current_position->si_uniq_info_ptr)) {
+		    if((info->di_d_port == port || info->di_s_port == port) && 
+		       !memcmp(&info->di_s_addr, &src6->sin6_addr, 16) &&
+		       !memcmp(&info->di_d_addr, &dst6->sin6_addr, 16)) {
+		      *tlsssl=current_position->si_tls_ssl_ptr;
+		      return(RETURN_OK);
+		    }
+		  }
+		  current_position = current_position->si_next_socket_info_ptr;
 		}
 	}
 
@@ -2462,7 +2530,7 @@ kSocketRegistUnixdSocketInfo(
 
 	/* pointer change for cast and complete uniq_info */
 	temp_unixd = (UnixdInfo*)new->si_uniq_info_ptr;
-	strcpy(temp_unixd->ui_sockpath, sockpath);
+	strcpy((char *)temp_unixd->ui_sockpath, (const char *)sockpath);
 
 	/*
 	 * generate general part of structure
@@ -2516,7 +2584,9 @@ kSocketRegistDataSocketInfo(
 	struct sockaddr_storage d_addr,
 	int (*parser)(long, unsigned char *, unsigned char **, short *),
 	short           socketid,
-	unsigned char   family)
+	unsigned char   family,
+	int		tls_mode,
+	void		*tls_ssl)
 {
 	char buf[BUFSIZE];
 	socklen_t buflen = sizeof(buf);
@@ -2594,9 +2664,11 @@ kSocketRegistDataSocketInfo(
 	new->si_socketid = g_socket_socketid;
 
 	new->si_sockethandle  = handle;
-	strcpy(new->si_linkname, linkname);
+	strcpy((char *)new->si_linkname, (const char *)linkname);
 	new->si_socket_status = SI_STATUS_ALIVE;
 	new->si_peyload_type  = peyload_type;
+	new->si_tls_mode      = tls_mode;
+	new->si_tls_ssl_ptr   = tls_ssl;
 
 	datasocketinfo->di_parser = parser;
 	datasocketinfo->di_parent_socketid = socketid;
@@ -2635,20 +2707,20 @@ kSocketRegistDataSocketInfo(
 
 	if (family == AF_INET) {
 		inaddr2string((struct in_addr *)&datasocketinfo->di_s_addr,
-			      buf, &buflen);
+			      buf, (int *)&buflen);
 		kLogWrite(L_DEBUG, "%s: srcaddr(%s)", __FUNCTION__, buf);
 
 		inaddr2string((struct in_addr *)&datasocketinfo->di_d_addr,
-			      buf, &buflen);
+			      buf, (int *)&buflen);
 		kLogWrite(L_DEBUG, "%s: dstaddr(%s)", __FUNCTION__, buf);
 	}
 	else {
 		in6addr2string((struct in6_addr *)&datasocketinfo->di_s_addr,
-			       buf, &buflen);
+			       buf, (int *)&buflen);
 		kLogWrite(L_DEBUG, "%s: srcaddr(%s)", __FUNCTION__, buf);
 
 		in6addr2string((struct in6_addr *)&datasocketinfo->di_d_addr,
-			       buf, &buflen);
+			       buf, (int *)&buflen);
 		kLogWrite(L_DEBUG, "%s: dstaddr(%s)", __FUNCTION__, buf);
 	}
 
@@ -2740,7 +2812,8 @@ kSocketCheckNewSocket(
 		if((si->si_type == SI_TYPE_UDP) &&
 		    (si->si_socket_status == SI_STATUS_ALIVE)) {
 			datasocketinfo = (DatasocketInfo*)si->si_uniq_info_ptr;
-			if(datasocketinfo->di_s_port == s_port) {
+/*			if(datasocketinfo->di_s_port == s_port) { 08/4/18 */
+			if(datasocketinfo->di_s_port == s_port && (!d_port || (datasocketinfo->di_d_port == d_port))) {
 				flg = false;
 				reuseinfo = si;
 
@@ -2964,7 +3037,8 @@ checksum(unsigned short *buf, int buflen)
 static int
 icmp_send(int fd, const unsigned char *linkname,
 	  struct sockaddr *src, socklen_t srclen,
-	  struct sockaddr *dst, socklen_t dstlen)
+	  struct sockaddr *dst, socklen_t dstlen,
+	  long datalen, unsigned char *data)
 {
 	int n;
 	u_char buf[MSG_MAXDATALEN];
@@ -2975,6 +3049,8 @@ icmp_send(int fd, const unsigned char *linkname,
 	struct iovec iov;
 	char control[1024];
 
+	char *scmsg = NULL;
+
 	memset(&iov, 0, sizeof(iov));
 	memset(control, 0, sizeof(control));
 
@@ -2984,63 +3060,77 @@ icmp_send(int fd, const unsigned char *linkname,
 	if (src->sa_family == AF_INET) {
 		struct icmp *icmp_header;
 		icmp_header = (struct icmp *) buf;
+		memset(icmp_header, 0, sizeof(struct icmp));
 
-		icmp_header->icmp_type = ICMP_ECHO;
-		icmp_header->icmp_code = 0;
+		if (datalen > 3) {
+			memcpy(icmp_header, data, 2);
+		}
+		else {
+			icmp_header->icmp_type = ICMP_ECHO;
+			icmp_header->icmp_code = 0;
+		}
 		icmp_header->icmp_cksum = 0;
 		icmp_header->icmp_id = 0;
 		icmp_header->icmp_seq = 0;
-		icmp_header->icmp_seq = 0;
 		memset(icmp_header->icmp_data, 0, 20);
 			
 		icmp_header->icmp_cksum =
 			checksum((unsigned short *)icmp_header,
-				 sizeof(struct icmp));
+				 sizeof(struct icmp)+20);
 
-		buflen = sizeof(struct icmp);
+		msg.msg_control = NULL;
+		msg.msg_controllen = 0;
+
+		buflen = sizeof(struct icmp) + 20;
 	}
 	else {
 		struct icmp6_hdr *icmp6_header = NULL;
 		int ip6optlen = 0;
 		struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *)src;
+		struct cmsghdr *scmsgp = NULL;
+		struct in6_pktinfo *pktinfo = NULL;
 
 		icmp6_header = (struct icmp6_hdr *) buf;
 
-		icmp6_header->icmp6_type = ICMP6_ECHO_REQUEST;
-		icmp6_header->icmp6_code = htons(0);
+		if (datalen > 3) {
+			/* ICMPv6 Type (1) + ICMPv6 Code (1) */
+			memcpy(icmp6_header, data, 2);
+		}
+		else {
+			icmp6_header->icmp6_type = ICMP6_ECHO_REQUEST;
+			icmp6_header->icmp6_code = htons(0);
+		}
 		/* Checksum is calculated by kernel in IPv6 */
 		icmp6_header->icmp6_cksum = htons(0);
 		icmp6_header->icmp6_id = htons(0);
 		icmp6_header->icmp6_seq = htons(0);
 
+		memcpy(data, icmp6_header, sizeof(struct icmp6_hdr));
+
+		ip6optlen += CMSG_SPACE(sizeof(struct in6_pktinfo));
+
+		if ((scmsg = (char *)malloc(ip6optlen)) == 0) {
+			fprintf(stderr, "can't allocate memory\n");
+		}
+
+		msg.msg_control = (caddr_t) scmsg;
+		msg.msg_controllen = ip6optlen;
+		scmsgp = (struct cmsghdr *)scmsg;
+
+		pktinfo = (struct in6_pktinfo *)(CMSG_DATA(scmsgp));
+
+		memset(pktinfo, 0, sizeof(*pktinfo));
+		scmsgp->cmsg_len = CMSG_LEN(sizeof(struct in6_pktinfo));
+		scmsgp->cmsg_level = IPPROTO_IPV6;
+		scmsgp->cmsg_type = IPV6_PKTINFO;
+		scmsgp = CMSG_NXTHDR(&msg, scmsgp);
+
+
+		memcpy(&pktinfo->ipi6_addr,
+		       &sin6->sin6_addr,
+		       sizeof(struct in6_addr));
+
 		if (IN6_IS_ADDR_LINKLOCAL(&sin6->sin6_addr)) {
-			/* use link-local address */
-			struct cmsghdr *scmsgp = NULL;
-			char *scmsg;
-			struct in6_pktinfo *pktinfo = NULL;
-
-			ip6optlen += CMSG_SPACE(sizeof(struct in6_pktinfo));
-
-			if ((scmsg = (char *)malloc(ip6optlen)) == 0) {
-				fprintf(stderr, "can't allocate memory\n");
-			}
-
-			msg.msg_control = (caddr_t) scmsg;
-			msg.msg_controllen = ip6optlen;
-			scmsgp = (struct cmsghdr *)scmsg;
-
-			pktinfo = (struct in6_pktinfo *)(CMSG_DATA(scmsgp));
-
-			memset(pktinfo, 0, sizeof(*pktinfo));
-			scmsgp->cmsg_len = CMSG_LEN(sizeof(struct in6_pktinfo));
-			scmsgp->cmsg_level = IPPROTO_IPV6;
-			scmsgp->cmsg_type = IPV6_PKTINFO;
-			scmsgp = CMSG_NXTHDR(&msg, scmsgp);
-
-			/* should modify hard coding */
-			memcpy(&pktinfo->ipi6_addr,
-			       &sin6->sin6_addr,
-			       sizeof(struct in6_addr));
 			pktinfo->ipi6_ifindex = linkname2ifindex(linkname);
 			if (pktinfo->ipi6_ifindex == 0) {
 				fprintf(stderr,
@@ -3066,6 +3156,9 @@ icmp_send(int fd, const unsigned char *linkname,
 		perror("sendmsg");
 	}
 
+	if (scmsg) {
+		free(scmsg);
+	}
 	kLogWrite(L_INFO, "%s: done", __FUNCTION__);
 	return n;
 }
@@ -3073,11 +3166,10 @@ icmp_send(int fd, const unsigned char *linkname,
 int
 icmp_recv(int fd, const unsigned char *linkname,
 	  struct sockaddr *src, socklen_t srclen,
-	  struct sockaddr *dst, socklen_t dstlen)
+	  struct sockaddr *dst, socklen_t dstlen,
+	  char *buf, int buflen)
 {
 	int n;
-	char buf[MSG_MAXDATALEN + 1];
-	int buflen = MSG_MAXDATALEN;
 
 	struct ip *ip_header_ptr;
 	struct icmp *icmp_header_ptr;
@@ -3197,7 +3289,9 @@ icmp_recv(int fd, const unsigned char *linkname,
 
 	kLogWrite(L_INFO, "%s: done", __FUNCTION__);
 
-	return true;
+	/* 8 = ICMPv6 Type (1) + ICMPv6 Code (1) + ICMPv6 Checksum (2) +
+	       ICMPv6 ID (2) + ICMPv6 Seq (2) */
+	return(8); 
 }
 
 int
@@ -3244,18 +3338,18 @@ kSocketOpenRaw(char af, struct in6_addr d_addr, struct in6_addr s_addr,
 	}
 
 	if(addrfamily == AF_INET) {
-		inaddr2string((struct in_addr *)&s_addr, buf, &buflen);
+		inaddr2string((struct in_addr *)&s_addr, buf, (int *)&buflen);
 		strcpy(srcaddrstr, buf);
-		inaddr2string((struct in_addr *)&d_addr, buf, &buflen);
+		inaddr2string((struct in_addr *)&d_addr, buf, (int *)&buflen);
 		strcpy(dstaddrstr, buf);
 
 		kLogWrite(L_INFO, "%s: src addr inet(%s)", __FUNCTION__, srcaddrstr);
 		kLogWrite(L_INFO, "%s: dst addr inet(%s)", __FUNCTION__, dstaddrstr);
 	}
 	else if(addrfamily == AF_INET6) {
-		in6addr2string(&s_addr, buf, &buflen);
+		in6addr2string(&s_addr, buf, (int *)&buflen);
 		strcpy(srcaddrstr, buf);
-		in6addr2string(&d_addr, buf, &buflen);
+		in6addr2string(&d_addr, buf, (int *)&buflen);
 		strcpy(dstaddrstr, buf);
 
 		kLogWrite(L_INFO, "%s: src addr inet6(%s)", __FUNCTION__, srcaddrstr);
@@ -3268,7 +3362,7 @@ kSocketOpenRaw(char af, struct in6_addr d_addr, struct in6_addr s_addr,
 		return(-1);
 	}
 
-	sockaddr2string((struct sockaddr *)&src, buf, &buflen);
+	sockaddr2string((struct sockaddr *)&src, buf, (int *)&buflen);
 
 	/* probe destination addr */
 	hints.ai_family = addrfamily;
@@ -3290,7 +3384,7 @@ kSocketOpenRaw(char af, struct in6_addr d_addr, struct in6_addr s_addr,
 	}
 	
 	buflen = sizeof(buf);
-	sockaddr2string(res0->ai_addr, buf, &buflen);
+	sockaddr2string(res0->ai_addr, buf, (int *)&buflen);
 
 	fd = socket(res0->ai_family, res0->ai_socktype, res0->ai_protocol);
 	if (fd < 0) {
@@ -3308,6 +3402,7 @@ kSocketOpenRaw(char af, struct in6_addr d_addr, struct in6_addr s_addr,
 		return(-1);
 	}
 
+/*
 	error = setsockopt(fd, SOL_SOCKET, SO_DONTROUTE,
 			   (char *)&on, sizeof(on));
 	if (error) {
@@ -3316,6 +3411,7 @@ kSocketOpenRaw(char af, struct in6_addr d_addr, struct in6_addr s_addr,
 		close(fd);
 		return(-1);
 	}
+*/
 
 	error = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
 			   &on, sizeof(on));
@@ -3490,7 +3586,7 @@ kSocketOpenRaw(char af, struct in6_addr d_addr, struct in6_addr s_addr,
 	socketInfo->si_socketid	= g_socket_socketid;
 
 	socketInfo->si_sockethandle = fd;
-	strcpy(socketInfo->si_linkname, linkname);
+	strcpy((char *)socketInfo->si_linkname, (const char *)linkname);
 	socketInfo->si_socket_status = SI_STATUS_ALIVE;
 	socketInfo->si_peyload_type = nullparser; /* P_NULL */
 
