@@ -1,6 +1,8 @@
 /*
- * Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005
- * Yokogawa Electric Corporation, YDC Corporation.  All rights reserved.
+ *
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010
+ * NTT Advanced Technology, Yokogawa Electric Corporation.
+ * All rights reserved.
  * 
  * Redistribution and use of this software in source and binary
  * forms, with or without modification, are permitted provided that
@@ -41,6 +43,9 @@
  * POSSIBILITY OF SUCH DAMAGE.
  *
  * $TAHI: koi/lib/kSocket/kTls.c,v 1.3 2007/04/05 07:54:17 akisada Exp $
+ *
+ * $Id: kTls.c,v 1.3 2008/07/14 02:17:02 inoue Exp $
+ *
  */
 
 #ifdef SUPPORT_TLS
@@ -62,6 +67,7 @@
 #include <fcntl.h>
 
 #include <signal.h>
+#include <netinet/tcp.h>
 #include <openssl/bio.h>
 #include <openssl/err.h>
 #include <openssl/rand.h>
@@ -97,7 +103,10 @@ static char *SSL_STATUS[] ={"SSL_ERROR_NONE","SSL_ERROR_SSL",
 			    "SSL_ERROR_WANT_X509_LOOKUP","SSL_ERROR_SYSCALL",
 			    "SSL_ERROR_ZERO_RETURN","SSL_ERROR_WANT_CONNECT",
 			    "SSL_ERROR_WANT_ACCEPT" ,"","",""};
+/* Nagle르ꥺ̵ */
+static int NagleFlag = 1;
 
+void TLSError(char *funcname,int err,SSL *ssl);
 int VerifyCallback(int ok, X509_STORE_CTX *store);
 /* TLSνλ */
 bool kIsTLSInitialize()
@@ -155,6 +164,7 @@ int kTLSRecv(void *tlsssl,char *buff,int buffsize)
       TLSClose(ssl,0);
       return 0;
     case SSL_ERROR_SYSCALL:
+      kLogWrite(L_WARNING,"%s: kTLSRecv SSL_read status[%s]",__FUNCTION__,SSL_STATUS[status]);
       TLSClose(ssl,0);
       return 0;
     case SSL_ERROR_WANT_READ:
@@ -182,9 +192,18 @@ void *kTLSConnect(int handle,void *tlsssl)
   SSL *ssl=(SSL*)tlsssl;
   X509 *peer;
   char peer_CN[256];
+  bool flg;
 
   if(!TLSctx){return NULL;}
   
+  /* Nagle 르ꥺ̵ˤ */
+  if(!NagleFlag){
+    flg=true;
+    if(setsockopt(handle,IPPROTO_TCP,TCP_NODELAY,(char*)&flg,sizeof(flg))){
+      kLogWrite(L_ERROR,"%s: Couldn't setsocktopt TCP_NODELAY",__FUNCTION__);
+    }
+  }
+
   /* åѲǽTLSåμ */
   if(TLSSessionMode && ssl) session=SSL_get_session(ssl);
 
@@ -228,6 +247,7 @@ void *kTLSConnect(int handle,void *tlsssl)
     X509_free(peer);
   }
 
+  kLogWrite(L_TLS,"%s: TLS Connect OK",__FUNCTION__);
   return (void*)ssl;
 }
 
@@ -236,6 +256,7 @@ int kTLSAccept(int handle,void **tlsssl,void **tlssession)
   BIO *bio;
   SSL *ssl;
   int ofcmode;
+  bool flg;
 
   if(!TLSctx){return(RETURN_NG);}
 
@@ -263,13 +284,23 @@ int kTLSAccept(int handle,void **tlsssl,void **tlssession)
     TLSClose(ssl,0);
     return(RETURN_NG);
   }
+  /* Nagle 르ꥺ̵ˤ */
+  if(!NagleFlag){
+    flg=true;
+    if(setsockopt(handle,IPPROTO_TCP,TCP_NODELAY,(char*)&flg,sizeof(flg))){
+      kLogWrite(L_ERROR,"%s: Couldn't setsocktopt TCP_NODELAY",__FUNCTION__);
+    }
+  }
+
   *tlsssl=(void*)ssl;
   *tlssession=(void*)SSL_get_session(ssl);
-  kLogWrite(L_TLS,"%s: Sub socket nonblocking OK",__FUNCTION__);
+  kLogWrite(L_TLS,"%s: TLS Accept socket nonblocking OK",__FUNCTION__);
   return(RETURN_OK);
 }
 
-int kTLSInitialize(int sessionMode,int initialmode,int timeout,char *passwd,char *rootPEM,char *myPEM,char *dhPEM,int version)
+/* ǥեȤξ ʬ:/usr/local/ct/cert/tn.pem ǧڶ:/usr/local/ct/cert/root.pem */
+int kTLSInitialize(int sessionMode,int initialmode,int timeout,char *passwd,char *rootPEM,char *myPEM,
+		   char *dhPEM,int version,int nagle,int clientveri,int tmprsa,char *enc)
 {
     BIO         *bio;
     SSL_METHOD *meth;
@@ -277,6 +308,9 @@ int kTLSInitialize(int sessionMode,int initialmode,int timeout,char *passwd,char
     DH *dh=0;
     struct stat tmp;
 
+    /* Nagle르ꥺ̵ե饰¸ */
+    NagleFlag=nagle;
+
     if(TLSctx){
       /* ⡼ɤ */
       if(initialmode){
@@ -290,10 +324,12 @@ int kTLSInitialize(int sessionMode,int initialmode,int timeout,char *passwd,char
     /* ѥ᡼μ */
     timeout = timeout<=0 ? SESSION_TIMEOUT : timeout;
     TLSSessionMode = sessionMode;
-    rootPEM = rootPEM ? rootPEM : ROOT_PEM;
-    myPEM   = myPEM ?   myPEM   : MY_PEM;
-    dhPEM   = dhPEM ?   dhPEM   : DHFILE1024;
-    strcpy(TLSPasswd,passwd ?  passwd  : PASSWORD);
+    kLogWrite(L_TLS,"%s: Session cache mode : %s   Session timeout : %d(s)",__FUNCTION__,sessionMode?"Enable":"Disable",timeout);
+
+    rootPEM = (rootPEM && rootPEM[0]) ? rootPEM : ROOT_PEM;
+    myPEM   = (myPEM && myPEM[0]) ?   myPEM   : MY_PEM;
+    dhPEM   = (dhPEM && dhPEM[0]) ?   dhPEM   : DHFILE1024;
+    strcpy(TLSPasswd,(passwd && passwd[0]) ?  passwd  : PASSWORD);
 
     if(lstat(rootPEM,&tmp)<0){
       kLogWrite(L_ERROR, "%s: TLS Initialize file[%s] not exist", __FUNCTION__,rootPEM);
@@ -366,7 +402,10 @@ int kTLSInitialize(int sessionMode,int initialmode,int timeout,char *passwd,char
     kLogWrite(L_TLS,"%s: SSL_CTX_load_verify_locations[%s] OK",__FUNCTION__,rootPEM);
 
     /* ǧڥХåؿϿ */
-    SSL_CTX_set_verify(TLSctx, SSL_VERIFY_PEER,VerifyCallback);
+    /* Server Key Exchange ץͿ (饤ǧڤԤ) */
+    kLogWrite(L_TLS,"%s: Server Key Exchange option : %s",__FUNCTION__,clientveri?"Enable":"Disable");
+    SSL_CTX_set_verify(TLSctx, clientveri?SSL_VERIFY_PEER:SSL_VERIFY_NONE,VerifyCallback);
+    kLogWrite(L_TLS,"%s:  SSL_CTX_set_verify[%s] OK",__FUNCTION__,clientveri?"PEER":"NONE");
     
     /* ǤãޤǤΥβꤹ */
     /* SSL_CTX_set_verify_depth(TLSctx,1); */
@@ -376,29 +415,45 @@ int kTLSInitialize(int sessionMode,int initialmode,int timeout,char *passwd,char
       SSL_CTX_set_options(TLSctx,SSL_OP_NO_SSLv2);
       kLogWrite(L_TLS,"%s: SSL_CTX_set_options NO_SSLv2",__FUNCTION__);
     }
-    /* BIO֥ȤȤäƥե򥪡ץ󤹤 */
-    if ((bio=BIO_new_file(dhPEM,"r")) == NULL){
-      TLSClose(NULL,0);
-      kLogWrite(L_ERROR, "%s: Couldn't open DH file",__FUNCTION__);
-      return(RETURN_NG);
+
+    /* Certificate Request ץͿ (ŪRSAȤ ) */
+    kLogWrite(L_TLS,"%s: Certificate Request option : %s",__FUNCTION__,tmprsa?"Enable":"Disable");
+    if(tmprsa){
+
+      /* BIO֥ȤȤäƥե򥪡ץ󤹤 */
+      if ((bio=BIO_new_file(dhPEM,"r")) == NULL){
+	TLSClose(NULL,0);
+	kLogWrite(L_ERROR, "%s: Couldn't open DH file",__FUNCTION__);
+	return(RETURN_NG);
+      }
+      kLogWrite(L_TLS,"%s: BIO_new_file[%s] OK",__FUNCTION__,dhPEM);
+    
+      /* ģȥѥ᡼ɤ߹ */
+      dh=PEM_read_bio_DHparams(bio,NULL,NULL,NULL);
+      kLogWrite(L_TLS,"%s: PEM_read_bio_DHparams[%s] OK",__FUNCTION__,dhPEM);
+      
+      /* ե򥯥 */
+      BIO_free(bio);
+    
+      /* ģȥѥ᡼CTX֥Ȥ˥ɤ */
+      if(SSL_CTX_set_tmp_dh(TLSctx,dh)<0){
+	TLSClose(NULL,0);
+	kLogWrite(L_ERROR, "%s: Couldn't set DH parameters",__FUNCTION__);
+	return(RETURN_NG);
+      }
+      kLogWrite(L_TLS,"%s: SSL_CTX_set_tmp_dh OK",__FUNCTION__);
     }
-    kLogWrite(L_TLS,"%s: BIO_new_file[%s] OK",__FUNCTION__,dhPEM);
-    
-    /* ģȥѥ᡼ɤ߹ */
-    dh=PEM_read_bio_DHparams(bio,NULL,NULL,NULL);
-    kLogWrite(L_TLS,"%s: PEM_read_bio_DHparams[%s] OK",__FUNCTION__,dhPEM);
-    
-    /* ե򥯥 */
-    BIO_free(bio);
-    
-    /* ģȥѥ᡼CTX֥Ȥ˥ɤ */
-    if(SSL_CTX_set_tmp_dh(TLSctx,dh)<0){
-      TLSClose(NULL,0);
-      kLogWrite(L_ERROR, "%s: Couldn't set DH parameters",__FUNCTION__);
-      return(RETURN_NG);
+
+    /* Ź楹Ȥ */
+    if(enc && enc[0]){
+      kLogWrite(L_TLS,"%s: Encrypt suit specified",__FUNCTION__);
+
+      if(SSL_CTX_set_cipher_list(TLSctx,enc))
+	kLogWrite(L_TLS,"%s:  SSL_CTX_set_cipher_list[%s] OK",__FUNCTION__,enc);
+      else
+	kLogWrite(L_TLS,"%s:  SSL_CTX_set_cipher_list[%s] invalid",__FUNCTION__,enc);
     }
-    kLogWrite(L_TLS,"%s: SSL_CTX_set_tmp_dh OK",__FUNCTION__);
-    
+
     /* ңӣΥڥ */
     rsa=RSA_generate_key(512,RSA_F4,NULL,NULL);
     kLogWrite(L_TLS,"%s: RSA_generate_key OK",__FUNCTION__);
@@ -421,6 +476,8 @@ int kTLSInitialize(int sessionMode,int initialmode,int timeout,char *passwd,char
     SSL_CTX_sess_set_remove_cb(TLSctx,remove_session_cb);
     SSL_CTX_set_timeout(TLSctx,timeout);
 
+    kLogWrite(L_TLS,"%s: Nagle algorithm : %s",__FUNCTION__,NagleFlag?"Enable":"Disable");
+
     return(RETURN_OK);
 }
 
@@ -516,9 +573,30 @@ int VerifyCallback(int ok, X509_STORE_CTX *store)
 	kLogWrite(L_ERROR, "%s:   depth: %i issuer: %s", __FUNCTION__, depth,data);
         X509_NAME_oneline(X509_get_subject_name(cert), data, 256);
 	kLogWrite(L_ERROR, "%s:   subject: %s", __FUNCTION__, data);
-        kLogWrite(L_ERROR, "%s:   %i:%s", __FUNCTION__, err, X509_verify_cert_error_string(err));
+        kLogWrite(L_ERROR, "%s:   ERR[%i]:%s", __FUNCTION__, err, X509_verify_cert_error_string(err));
     }
  
     return ok;
 }
+
+void TLSError(char *funcname,int err,SSL *ssl)
+{
+  int flags,line;
+  char buff[256];
+  const char *data,*file;
+  unsigned long code;
+
+  ERR_error_string_n(SSL_get_error(ssl,err),buff,sizeof(buff));
+  kLogWrite(L_ERROR,"%s: %s",funcname,buff);
+  return;
+
+  code = ERR_get_error_line_data(&file,&line,&data,&flags);
+  while(code){
+    if(data && (flags & ERR_TXT_STRING))
+      kLogWrite(L_ERROR,"%s: %s",funcname,data);
+    code = ERR_get_error_line_data(&file,&line,&data,&flags);
+  }
+}
+
+
 #endif	/* SUPPORT_TLS */
