@@ -1,7 +1,7 @@
 #!/usr/bin/perl -w
 #
-# Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
-# Yokogawa Electric Corporation.
+# Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010
+# NTT Advanced Technology, Yokogawa Electric Corporation.
 # All rights reserved.
 # 
 # Redistribution and use of this software in source and binary
@@ -44,6 +44,8 @@
 #
 # $TAHI: koi/libdata/kRemote/kRemote.pm,v 1.8 2006/06/30 07:10:27 akisada Exp $
 #
+# $Id: kRemote.pm,v 1.4 2008/06/03 07:40:00 akisada Exp $
+#
 
 package kRemote;
 
@@ -214,6 +216,10 @@ sub kRemoteInit(;$$)
 		$prompt_password{$System} = $NutDef{'PasswordPrompt'};
 	}
 
+	if (exists($NutDef{'EscapePrompt'})) {
+		$prompt_escape{$System} = $NutDef{'EscapePrompt'};
+	}
+
 	pr("tn.def...\n");
 	foreach my $key (sort(keys(%TnDef))) {
 		pr("\t$key => $TnDef{$key}\n");
@@ -232,6 +238,10 @@ sub kRemoteInit(;$$)
 		$prompt_password{$System} = $prompt_password{'default'};
 	}
 
+	unless (exists($prompt_escape{$System})) {
+		$prompt_escape{$System} = $prompt_escape{'default'};
+	}
+
 	unless (exists($prompt_command{$System})) {
 		$prompt_command{$System} = $prompt_command{'default'};
 	}
@@ -520,7 +530,7 @@ kRemoteWriteFile($$;@)
 		return(undef);
 	}
 
-	unless(defined(sendMessagesSync("cat > $file << 'EOF'\r"))) {
+	unless(defined(sendMessagesSync("cat > $file << EOF\r"))) {
 		prErr("$_function_: sendMessagesSync() failure\n");
 		return(undef);
 	}
@@ -542,7 +552,7 @@ kRemoteWriteFile($$;@)
 		return(undef);
 	}
 
-	unless(defined(sendMessagesSync("'EOF'\r"))) {
+	unless(defined(sendMessagesSync("EOF\r"))) {
 		prErr("$_function_: sendMessagesSync() failure\n");
 		return(undef);
 	}
@@ -757,6 +767,9 @@ sub readNutDef($)
 		elsif ($name eq 'PasswordPrompt') {
 			$prompt_password{$NutDef{'System'}} = $value;
 		}
+		elsif ($name eq 'EscapePrompt') {
+			$prompt_escape{$NutDef{'System'}} = $value;
+		}
 		elsif (/^(Link[0-9]+)\s+(\S+)/) {
 			$TnDef{$name."_device"} = $value;
 		}
@@ -947,7 +960,7 @@ waitQuestionMark()
 
 	for($i=0; ; $i++) {
 		my $ret = $expect->expect($timeout,
-					  '-re', qr'\?');
+					  '-re', qr"$prompt_escape{$System}");
 		if(defined($ret)) {
 			pr("$_function_: got command prompt");
 			last;
@@ -1063,6 +1076,10 @@ BEGIN {
 		'default' => '^[P|p]assword:',
 	);
 
+	%prompt_escape = (
+		'default' => '\?',
+	);
+
 	%prompt_command = (
 		'default' => '[$#>:] $',
 	);
